@@ -5,7 +5,7 @@
  | program/include/rcube_mdb2.php                                        |
  |                                                                       |
  | This file is part of the Roundcube Webmail client                     |
- | Copyright (C) 2005-2009, Roundcube Dev. - Switzerland                 |
+ | Copyright (C) 2005-2009, The Roundcube Dev Team                       |
  | Licensed under the GNU GPL                                            |
  |                                                                       |
  | PURPOSE:                                                              |
@@ -16,7 +16,7 @@
  | Author: Lukas Kahwe Smith <smith@pooteeweet.org>                      |
  +-----------------------------------------------------------------------+
 
- $Id: rcube_mdb2.php 3993 2010-09-26 11:17:03Z thomasb $
+ $Id: rcube_mdb2.php 5543 2011-12-05 07:24:36Z alec $
 
 */
 
@@ -30,20 +30,21 @@
  * @author     David Saez Padros <david@ols.es>
  * @author     Thomas Bruederli <roundcube@gmail.com>
  * @author     Lukas Kahwe Smith <smith@pooteeweet.org>
- * @version    1.17
+ * @version    1.18
  * @link       http://pear.php.net/package/MDB2
  */
 class rcube_mdb2
 {
-    var $db_dsnw;               // DSN for write operations
-    var $db_dsnr;               // DSN for read operations
-    var $db_connected = false;  // Already connected ?
-    var $db_mode = '';          // Connection mode
-    var $db_handle = 0;         // Connection handle
-    var $db_error = false;
-    var $db_error_msg = '';
+    public $db_dsnw;               // DSN for write operations
+    public $db_dsnr;               // DSN for read operations
+    public $db_connected = false;  // Already connected ?
+    public $db_mode = '';          // Connection mode
+    public $db_handle = 0;         // Connection handle
+    public $db_error = false;
+    public $db_error_msg = '';
 
     private $debug_mode = false;
+    private $conn_failure = false;
     private $a_query_results = array('dummy');
     private $last_res_id = 0;
     private $tables;
@@ -52,12 +53,12 @@ class rcube_mdb2
     /**
      * Object constructor
      *
-     * @param  string  DSN for read/write operations
-     * @param  string  Optional DSN for read only operations
+     * @param  string $db_dsnw DSN for read/write operations
+     * @param  string $db_dsnr Optional DSN for read only operations
      */
     function __construct($db_dsnw, $db_dsnr='', $pconn=false)
     {
-        if ($db_dsnr == '')
+        if (empty($db_dsnr))
             $db_dsnr = $db_dsnw;
 
         $this->db_dsnw = $db_dsnw;
@@ -72,8 +73,8 @@ class rcube_mdb2
     /**
      * Connect to specific database
      *
-     * @param  string  DSN for DB connections
-     * @return object  PEAR database handle
+     * @param  string $dsn  DSN for DB connections
+     * @return MDB2 PEAR database handle
      * @access private
      */
     private function dsn_connect($dsn)
@@ -83,13 +84,15 @@ class rcube_mdb2
             'persistent'       => $this->db_pconn,
             'emulate_prepared' => $this->debug_mode,
             'debug'            => $this->debug_mode,
-            'debug_handler'    => 'mdb2_debug_handler',
+            'debug_handler'    => array($this, 'debug_handler'),
             'portability'      => MDB2_PORTABILITY_ALL ^ MDB2_PORTABILITY_EMPTY_TO_NULL);
 
         if ($this->db_provider == 'pgsql') {
             $db_options['disable_smart_seqname'] = true;
             $db_options['seqname_format'] = '%s';
         }
+        $this->db_error     = false;
+        $this->db_error_msg = null;
 
         $dbh = MDB2::connect($dsn, $db_options);
 
@@ -116,40 +119,52 @@ class rcube_mdb2
     /**
      * Connect to appropiate database depending on the operation
      *
-     * @param  string  Connection mode (r|w)
+     * @param  string $mode Connection mode (r|w)
      * @access public
      */
     function db_connect($mode)
     {
+        // previous connection failed, don't attempt to connect again
+        if ($this->conn_failure) {
+            return;
+        }
+
+        // no replication
+        if ($this->db_dsnw == $this->db_dsnr) {
+            $mode = 'w';
+        }
+
         // Already connected
         if ($this->db_connected) {
-            // connected to read-write db, current connection is ok
-            if ($this->db_mode == 'w')
-                return;
-
-            // no replication, current connection is ok for read and write
-            if (empty($this->db_dsnr) || $this->db_dsnw == $this->db_dsnr) {
-                $this->db_mode = 'w';
+            // connected to db with the same or "higher" mode
+            if ($this->db_mode == 'w' || $this->db_mode == $mode) {
                 return;
             }
-
-            // Same mode, current connection is ok
-            if ($this->db_mode == $mode)
-                return;
         }
 
         $dsn = ($mode == 'r') ? $this->db_dsnr : $this->db_dsnw;
 
-        $this->db_handle = $this->dsn_connect($dsn);
+        $this->db_handle    = $this->dsn_connect($dsn);
         $this->db_connected = !PEAR::isError($this->db_handle);
-        $this->db_mode = $mode;
+
+        // use write-master when read-only fails
+        if (!$this->db_connected && $mode == 'r') {
+            $mode = 'w';
+            $this->db_handle    = $this->dsn_connect($this->db_dsnw);
+            $this->db_connected = !PEAR::isError($this->db_handle);
+        }
+
+        if ($this->db_connected)
+            $this->db_mode = $mode;
+        else
+            $this->conn_failure = true;
     }
 
 
     /**
      * Activate/deactivate debug mode
      *
-     * @param boolean True if SQL queries should be logged
+     * @param boolean $dbg True if SQL queries should be logged
      * @access public
      */
     function set_debug($dbg = true)
@@ -186,6 +201,16 @@ class rcube_mdb2
     }
 
 
+    /**
+     * Is database replication configured?
+     * This returns true if dsnw != dsnr
+     */
+    function is_replicated()
+    {
+      return !empty($this->db_dsnr) && $this->db_dsnw != $this->db_dsnr;
+    }
+
+
     /**
      * Execute a SQL query
      *
@@ -199,6 +224,10 @@ class rcube_mdb2
         $params = func_get_args();
         $query = array_shift($params);
 
+        // Support one argument of type array, instead of n arguments
+        if (count($params) == 1 && is_array($params[0]))
+            $params = $params[0];
+
         return $this->_query($query, 0, 0, $params);
     }
 
@@ -227,10 +256,10 @@ class rcube_mdb2
     /**
      * Execute a SQL query with limits
      *
-     * @param  string  SQL query to execute
-     * @param  number  Offset for LIMIT statement
-     * @param  number  Number of rows for LIMIT statement
-     * @param  array   Values to be inserted in query
+     * @param  string $query   SQL query to execute
+     * @param  number $offset  Offset for LIMIT statement
+     * @param  number $numrows Number of rows for LIMIT statement
+     * @param  array  $params  Values to be inserted in query
      * @return number  Query handle identifier
      * @access private
      */
@@ -262,7 +291,9 @@ class rcube_mdb2
 
                 raise_error(array('code' => 500, 'type' => 'db',
                     'line' => __LINE__, 'file' => __FILE__,
-                    'message' => $this->db_error_msg), true, true);
+                    'message' => $this->db_error_msg), true, false);
+
+                $result = false;
             }
             else {
                 $result = $q->execute($params);
@@ -279,13 +310,13 @@ class rcube_mdb2
      * Get number of rows for a SQL query
      * If no query handle is specified, the last query will be taken as reference
      *
-     * @param  number  Optional query handle identifier
+     * @param  number $res_id  Optional query handle identifier
      * @return mixed   Number of rows or false on failure
      * @access public
      */
     function num_rows($res_id=null)
     {
-        if (!$this->db_handle)
+        if (!$this->db_connected)
             return false;
 
         if ($result = $this->_get_result($res_id))
@@ -298,16 +329,16 @@ class rcube_mdb2
     /**
      * Get number of affected rows for the last query
      *
-     * @param  number  Optional query handle identifier
+     * @param  number $res_id Optional query handle identifier
      * @return mixed   Number of rows or false on failure
      * @access public
      */
     function affected_rows($res_id = null)
     {
-        if (!$this->db_handle)
+        if (!$this->db_connected)
             return false;
 
-        return (int) $this->_get_result($res_id);
+        return $this->_get_result($res_id);
     }
 
 
@@ -315,13 +346,13 @@ class rcube_mdb2
      * Get last inserted record ID
      * For Postgres databases, a sequence name is required
      *
-     * @param  string  Table name (to find the incremented sequence)
+     * @param  string $table  Table name (to find the incremented sequence)
      * @return mixed   ID or false on failure
      * @access public
      */
     function insert_id($table = '')
     {
-        if (!$this->db_handle || $this->db_mode == 'r')
+        if (!$this->db_connected || $this->db_mode == 'r')
             return false;
 
         if ($table) {
@@ -343,7 +374,7 @@ class rcube_mdb2
      * Get an associative array for one row
      * If no query handle is specified, the last query will be taken as reference
      *
-     * @param  number  Optional query handle identifier
+     * @param  number $res_id Optional query handle identifier
      * @return mixed   Array with col values or false on failure
      * @access public
      */
@@ -358,7 +389,7 @@ class rcube_mdb2
      * Get an index array for one row
      * If no query handle is specified, the last query will be taken as reference
      *
-     * @param  number  Optional query handle identifier
+     * @param  number $res_id  Optional query handle identifier
      * @return mixed   Array with col values or false on failure
      * @access public
      */
@@ -372,8 +403,8 @@ class rcube_mdb2
     /**
      * Get col values for a result row
      *
-     * @param  object  Query result handle
-     * @param  number  Fetch mode identifier
+     * @param  MDB2_Result_Common Query $result result handle
+     * @param  number                   $mode   Fetch mode identifier
      * @return mixed   Array with col values or false on failure
      * @access private
      */
@@ -408,11 +439,28 @@ class rcube_mdb2
     }
 
 
+    /**
+     * Wrapper for SHOW COLUMNS command
+     *
+     * @param string Table name
+     * @return array List of table cols
+     */
+    function list_cols($table)
+    {
+        $this->db_handle->loadModule('Manager');
+        if (!PEAR::isError($result = $this->db_handle->listTableFields($table))) {
+            return $result;
+        }
+
+        return null;
+    }
+
+
     /**
      * Formats input so it can be safely used in a query
      *
-     * @param  mixed   Value to quote
-     * @param  string  Type of data
+     * @param  mixed  $input  Value to quote
+     * @param  string $type   Type of data
      * @return string  Quoted/converted string for use in query
      * @access public
      */
@@ -426,14 +474,14 @@ class rcube_mdb2
         if (!$this->db_handle)
             $this->db_connect('r');
 
-        return $this->db_handle->quote($input, $type);
+        return $this->db_connected ? $this->db_handle->quote($input, $type) : addslashes($input);
     }
 
 
     /**
      * Quotes a string so it can be safely used as a table or column name
      *
-     * @param  string  Value to quote
+     * @param  string $str Value to quote
      * @return string  Quoted string for use in query
      * @deprecated     Replaced by rcube_MDB2::quote_identifier
      * @see            rcube_mdb2::quote_identifier
@@ -448,7 +496,7 @@ class rcube_mdb2
     /**
      * Quotes a string so it can be safely used as a table or column name
      *
-     * @param  string  Value to quote
+     * @param  string $str Value to quote
      * @return string  Quoted string for use in query
      * @access public
      */
@@ -457,14 +505,14 @@ class rcube_mdb2
         if (!$this->db_handle)
             $this->db_connect('r');
 
-        return $this->db_handle->quoteIdentifier($str);
+        return $this->db_connected ? $this->db_handle->quoteIdentifier($str) : $str;
     }
 
 
     /**
      * Escapes a string
      *
-     * @param  string  The string to be escaped
+     * @param  string $str The string to be escaped
      * @return string  The escaped string
      * @access public
      * @since  0.1.1
@@ -486,7 +534,7 @@ class rcube_mdb2
      */
     function now()
     {
-        switch($this->db_provider) {
+        switch ($this->db_provider) {
             case 'mssql':
             case 'sqlsrv':
                 return "getdate()";
@@ -500,8 +548,8 @@ class rcube_mdb2
     /**
      * Return list of elements for use with SQL's IN clause
      *
-     * @param  array  Input array
-     * @param  string Type of data
+     * @param  array  $arr  Input array
+     * @param  string $type Type of data
      * @return string Comma-separated list of quoted values for use in query
      * @access public
      */
@@ -520,9 +568,12 @@ class rcube_mdb2
     /**
      * Return SQL statement to convert a field value into a unix timestamp
      *
-     * @param  string  Field name
+     * This method is deprecated and should not be used anymore due to limitations
+     * of timestamp functions in Mysql (year 2038 problem)
+     *
+     * @param  string $field Field name
      * @return string  SQL statement to use in query
-     * @access public
+     * @deprecated
      */
     function unixtimestamp($field)
     {
@@ -543,29 +594,21 @@ class rcube_mdb2
     /**
      * Return SQL statement to convert from a unix timestamp
      *
-     * @param  string  Field name
+     * @param  string $timestamp Field name
      * @return string  SQL statement to use in query
      * @access public
      */
     function fromunixtime($timestamp)
     {
-        switch($this->db_provider) {
-            case 'mysqli':
-            case 'mysql':
-            case 'sqlite':
-                return sprintf("FROM_UNIXTIME(%d)", $timestamp);
-
-            default:
-                return date("'Y-m-d H:i:s'", $timestamp);
-        }
+        return date("'Y-m-d H:i:s'", $timestamp);
     }
 
 
     /**
      * Return SQL statement for case insensitive LIKE
      *
-     * @param  string  Field name
-     * @param  string  Search value
+     * @param  string $column  Field name
+     * @param  string $value   Search value
      * @return string  SQL statement to use in query
      * @access public
      */
@@ -580,11 +623,39 @@ class rcube_mdb2
         }
     }
 
+    /**
+     * Abstract SQL statement for value concatenation
+     *
+     * @return string SQL statement to be used in query
+     * @access public
+     */
+    function concat(/* col1, col2, ... */)
+    {
+        $func = '';
+        $args = func_get_args();
+
+        switch($this->db_provider) {
+            case 'mysql':
+            case 'mysqli':
+                $func = 'CONCAT';
+                $delim = ', ';
+                break;
+            case 'mssql':
+            case 'sqlsrv':
+                $delim = ' + ';
+                break;
+            default:
+                $delim = ' || ';
+        }
+
+        return $func . '(' . join($delim, $args) . ')';
+    }
+
 
     /**
      * Encodes non-UTF-8 characters in string/array/object (recursive)
      *
-     * @param  mixed  Data to fix
+     * @param  mixed  $input Data to fix
      * @return mixed  Properly UTF-8 encoded data
      * @access public
      */
@@ -608,7 +679,7 @@ class rcube_mdb2
     /**
      * Decodes encoded UTF-8 string/object/array (recursive)
      *
-     * @param  mixed  Input data
+     * @param  mixed $input Input data
      * @return mixed  Decoded data
      * @access public
      */
@@ -632,7 +703,7 @@ class rcube_mdb2
     /**
      * Adds a query result and returns a handle ID
      *
-     * @param  object  Query handle
+     * @param  object $res Query handle
      * @return mixed   Handle ID
      * @access private
      */
@@ -660,7 +731,7 @@ class rcube_mdb2
      * Resolves a given handle ID and returns the according query handle
      * If no ID is specified, the last resource handle will be returned
      *
-     * @param  number  Handle ID
+     * @param  number $res_id Handle ID
      * @return mixed   Resource handle or false on failure
      * @access private
      */
@@ -680,8 +751,8 @@ class rcube_mdb2
     /**
      * Create a sqlite database from a file
      *
-     * @param  object  SQLite database handle
-     * @param  string  File path to use for DB creation
+     * @param  MDB2   $dbh       SQLite database handle
+     * @param  string $file_name File path to use for DB creation
      * @access private
      */
     private function _sqlite_create_database($dbh, $file_name)
@@ -707,7 +778,7 @@ class rcube_mdb2
      */
     private function _sqlite_prepare()
     {
-        include_once('include/rcube_sqlite.inc');
+        include_once(INSTALL_PATH . 'program/include/rcube_sqlite.inc');
 
         // we emulate via callback some missing MySQL function
         sqlite_create_function($this->db_handle->connection,
@@ -720,16 +791,17 @@ class rcube_mdb2
             'md5', 'rcube_sqlite_md5');
     }
 
-}  // end class rcube_db
 
-
-/* this is our own debug handler for the MDB2 connection */
-function mdb2_debug_handler(&$db, $scope, $message, $context = array())
-{
-    if ($scope != 'prepare') {
-        $debug_output = sprintf('%s(%d): %s;',
-            $scope, $db->db_index, rtrim($message, ';'));
-        write_log('sql', $debug_output);
+    /**
+     * Debug handler for the MDB2
+     */
+    function debug_handler(&$db, $scope, $message, $context = array())
+    {
+        if ($scope != 'prepare') {
+            $debug_output = sprintf('%s(%d): %s;',
+                $scope, $db->db_index, rtrim($message, ';'));
+            write_log('sql', $debug_output);
+        }
     }
-}
 
+}  // end class rcube_db
